# Copyright (c) 2020 Jerome Arbez-Gindre
# frozen_string_literal: true

module DefMastership
  # Change references from temporary to definitive with multiple RefChangers
  class ChangeRefLineModifier < LineModifierBase
    def self.from_config(hash)
      new.from_config(hash)
    end

    REGEXP_FROM = {
      definition: {
        before: DMRegexp::DEF_BEFORE_REF,
        after: DMRegexp::DEF_AFTER_REF
      },
      iref: {
        before: DMRegexp::IREF_DEF_BEF,
        after: DMRegexp::IREF_DEF_AFT
      }
    }.freeze

    private_constant :REGEXP_FROM

    def initialize
      super
      @config = {
        from_regexp: '',
        to_template: '',
        next_ref: 0
      }
      @parsing_state = ParsingState.new
    end

    def replace(method, line)
      public_send("replace_#{method}".to_sym, line)
    end

    def replace_refdef(line)
      if @parsing_state.enabled?(line)
        do_replace_refdef(line)
      else
        line
      end
    end

    def replace_irefs(line)
      changes.reduce(line) do |res_line, (from, to)|
        res_line.gsub(regexp_from(:iref, from)) do
          text_with(Regexp.last_match, to)
        end
      end
    end

    private

    def do_replace_refdef(line)
      line.gsub(regexp_from(:definition, from_regexp)) do
        replacement = to_template % hmerge(Regexp.last_match)
        changes << [Regexp.last_match[:from], replacement]
        @config[:next_ref] += 1
        text_with(Regexp.last_match, replacement)
      end
    end

    def regexp_from(const, from)
      regexp_str =
        "(?<before>#{REGEXP_FROM[const][:before]})" \
        "(?<from>#{from})" \
        "#{DMRegexp::DEF_VERSION_AND_CHECKSUM}" \
        "(?<after>#{REGEXP_FROM[const][:after]})"
      Regexp.new(regexp_str, Regexp::EXTENDED)
    end

    def text_with(match, replacement)
      match[:before] + replacement + (match[:version_and_checksum] || '') + match[:after]
    end

    def hmerge(match)
      @config.merge(match.names.map(&:to_sym).zip(match.captures).to_h)
    end
  end
end
